import jcsp.lang.CSProcess;
import jcsp.lang.ChannelOutput;

/**
   This class was adapted from the version found on
   http://www.cs.kent.ac.uk/projects/ofa/java-threads/0.html.
   
   This represents a Chef who prepares chickens (in batches of 4)
   and brings them to the canteen for the philosophers.
   
   Note that the Chef is no longer a Thread.  The only part of this
   class that deals with synchronization issues is the call to the
   JCSP ChannelOutput object.
   
   @author taustin
 */
public class Chef implements CSProcess
{
   //Canteen where the Chef puts his chickens.
   //private Canteen canteen;
   private ChannelOutput canteenChannel;
   
   //The number of chickens that the Chef makes at a time.
   private final static int chickenBatchSize = 4;
  
   /**
     Constructor.
     
     @param canteen  Canteen where the chef puts his chickens.
    */
   public Chef (ChannelOutput canteenChannel)
   {
      this.canteenChannel = canteenChannel;
   }
   
   /**
      The Chef will cook chickens and then bring them to the canteen.
    */
   public void run ()
   {
      System.out.println (System.currentTimeMillis() + " - Chef  : starting ... ");
      while (true)
      {
         //cook chickens, taking 2 seconds total.
         System.out.println (System.currentTimeMillis() + " - Chef  : cooking ... ");
         try {Thread.sleep (2000);} catch (InterruptedException e) {}

         System.out.println (System.currentTimeMillis() + " - Chef  : "
                 + chickenBatchSize + " chickens, ready-to-go ... ");
         
         //The chef puts numbers on the channel one at a time
         // to simulate loading up the canteen.
         for (int i=0; i<chickenBatchSize; i++)
            canteenChannel.write(i);
      }
   }
}