import jcsp.lang.CSProcess;
import jcsp.lang.ChannelInput;

/**
   This class was adapted from http://www.cs.kent.ac.uk/projects/ofa/java-threads/0.html.
   This illustrates a solution to the Greedy Philosopher Problem by the use of
   the J-CSP library.
   
   Philosophers think and go to eat at the canteen, unless they are greedy, in which case
   they just eat at the canteen.
   
   Note that Philosopher no longer extends the Thread class.  The only part of this
   class that deals with synchronization issues is the call to the
   JCSP ChannelOutput object.

   @author taustin
 */
public class Philosopher implements CSProcess
{
   //ID of the Philosopher.
   private int id;
   
   //Canteen where the philosopher gets chickens.
   //private Canteen canteen;
   private ChannelInput canteenChannel;
   
   //Greedy philosophers do not waste time thinking and
   // go straight to the canteen.
   private boolean greedy;
  
   /**
      Constructor.
      
      @param id  ID number to identify the philosopher.
      @param canteenChannel  Represents the canteen where the philosopher eats.
      @param greedy  Is this professor greedy or not.
    */
   public Philosopher (int id, ChannelInput canteenChannel, boolean greedy)
   {
      this.id = id;
      this.canteenChannel = canteenChannel;
      this.greedy = greedy;
   }

   /**
      A good philosopher will spend some time thinking, and then will
      go to the canteen to eat.  A greedy philosopher will rush straight
      to the canteen.
    */
   public void run()
   {  
      System.out.println (System.currentTimeMillis() + " - Phil " + id + ": starting ... ");
      while (true)
      {
         //The greedy philosopher does not spend time thinking, but the
         // others do for 3 seconds.
         if (!greedy) 
         {
            try {Thread.sleep (3000);}
            catch (InterruptedException e) {}
         }
         System.out.println (System.currentTimeMillis() + " - Phil " + id + ": gotta eat ... ");

         //The Philosopher reads an Object from the queue to simulate eating a chicken.
         Object chicken = canteenChannel.read();
    
         System.out.println (System.currentTimeMillis() + " - Phil " + id
               + ":    mmm ... that's good -- eating chicken #" + chicken);
      }
   }
}